"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const jest_tracker_1 = __importDefault(require("./jest_rspec_styles/jest_tracker"));
const lazy_variable_1 = __importDefault(require("./jest_rspec_styles/lazy_variable"));
const shared_example_1 = __importDefault(require("./jest_rspec_styles/shared_example"));
const mock_1 = __importDefault(require("./jest_rspec_styles/mock"));
const matcher_1 = __importDefault(require("./jest_rspec_styles/matcher"));
/**
 * RSpec style extension for jest.
 *
 * @example
 * import JestRSpecStyle from 'jest-rspec-style';
 * JestRSpecStyle.setup();
 */
class JestRSpecStyle {
    /**
     * Setup plugin.
     */
    static setup() {
        const tracker = new jest_tracker_1.default();
        this._lazy = new lazy_variable_1.default(tracker);
        this._sharedExample = new shared_example_1.default();
        this._mock = new mock_1.default();
        this._matcher = new matcher_1.default(this._mock);
        global.context = global.describe;
        global.lazy = this._lazy.lazy.bind(this._lazy);
        global.subject = this._lazy.subject.bind(this._lazy);
        global.sharedExamples = this._sharedExample.sharedExamples.bind(this._sharedExample);
        global.includeExamples = this._sharedExample.includeExamples.bind(this._sharedExample);
        global.allow = this._mock.allow.bind(this._mock);
        global.allowAnyInstanceOf = this._mock.allowAnyInstanceOf.bind(this._mock);
        global.expectAnyInstanceOf = this._mock.expectAnyInstanceOf.bind(this._mock);
        this._matcher.setup();
        beforeEach(this._reset.bind(this));
        afterEach(this._verify.bind(this));
    }
    /**
     * Reset status.
     */
    static _reset() {
        this._lazy.reset();
        this._mock.reset();
    }
    /**
     * Verify status.
     */
    static _verify() {
        this._mock.verify();
    }
}
exports.default = JestRSpecStyle;
//# sourceMappingURL=jest_rspec_style.js.map