"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const definition_1 = __importDefault(require("./lazy_variables/definition"));
/**
 * Lazy Variable.
 */
class LazyVariable {
    /**
     * Initialize.
     * @param {JestTracker} tracker tracker of jest
     */
    constructor(tracker) {
        this._definitions = [];
        this._tracker = tracker;
        this.reset();
    }
    /**
     * Reset status.
     */
    reset() {
        this._caches = {};
    }
    /**
     * Create lazy variable.
     * @param {string}   name      name of variable.
     * @param {Function} evaluator function of variable evaluator. (If omit, get lazy variable.)
     * @return {Object} evaluated value.
     */
    lazy(name, evaluator) {
        const suite = this._tracker.getCurrentSuite();
        if (evaluator) {
            this._definitions.push(new definition_1.default(name, evaluator, suite.id));
            return;
        }
        const cache = this._caches[name];
        if (cache) {
            return cache;
        }
        let cursor = suite;
        while (cursor) {
            const definition = this._definitions.find((d) => d.suiteId == cursor.id && d.name == name);
            if (definition) {
                const value = definition.eval();
                this._caches[name] = value;
                return value;
            }
            cursor = cursor.parentSuite;
        }
        throw new Error(`lazy('${name}') is not defined.`);
    }
    /**
     * Create subject of test.
     * @param {Function} evaluator function of variable evaluator. (If omit, get lazy variable.)
     * @return {Object} evaluated value.
     */
    subject(evaluator) {
        return this.lazy('subject', evaluator);
    }
}
exports.default = LazyVariable;
//# sourceMappingURL=lazy_variable.js.map