"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Definition of lazy variable.
 */
class Definition {
    /**
     * Initialize
     * @param {string}   name      name of variable.
     * @param {Function} evaluator function of evaluation.
     * @param {string}   suiteId   id of suite.
     */
    constructor(name, evaluator, suiteId) {
        this._name = name;
        this._evaluator = evaluator;
        this._suiteId = suiteId;
    }
    /**
     * Get a name of variable.
     * @return {String} name of variable.
     */
    get name() {
        return this._name;
    }
    /**
     * Get a id of suite.
     * @return {String} id of suite.
     */
    get suiteId() {
        return this._suiteId;
    }
    /**
     * Evaluate lazy variable.
     * @return {any} evaluated value.
     */
    eval() {
        return this._evaluator();
    }
}
exports.default = Definition;
//# sourceMappingURL=definition.js.map