"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable @typescript-eslint/explicit-module-boundary-types */
const expected_stub_chain_1 = __importDefault(require("./mocks/expected_stub_chain"));
const stub_chain_1 = __importDefault(require("./mocks/stub_chain"));
/**
 * Stub
 */
class Mock {
    constructor() {
        this._stubChains = [];
    }
    /**
     * Stub object.
     * @param {Object} target Stub target.
     * @return {StubChain} Stub hain.
     */
    allow(target) {
        const stub = new stub_chain_1.default(target);
        this._stubChains.push(stub);
        return stub;
    }
    /**
     * Stub all instance of object.
     * @param {Object} target Stub target.
     * @return {StubChain} Stub hain.
     */
    allowAnyInstanceOf(target) {
        return this.allow(target.prototype);
    }
    /**
     * Expect and stub object.
     * @param {Object} target Stub target.
     * @return {StubChain} Stub hain.
     */
    expect(target) {
        const stub = new expected_stub_chain_1.default(target);
        this._stubChains.push(stub);
        return stub;
    }
    /**
     * Expect and stub all instance of object.
     * @param {Object} target Stub target.
     * @return {StubChain} Stub hain.
     */
    expectAnyInstanceOf(target) {
        return this.expect(target.prototype);
    }
    /**
     * Reet stubs.
     */
    reset() {
        this._stubChains = [];
        jest.restoreAllMocks();
    }
    /**
     * Verify stub chains.
     */
    verify() {
        this._stubChains.forEach((stub) => stub.verify());
    }
}
exports.default = Mock;
//# sourceMappingURL=mock.js.map