"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable @typescript-eslint/explicit-module-boundary-types */
const stub_chain_1 = __importDefault(require("./stub_chain"));
/**
 * Expected call stub chain.
 */
class ExpectedStubChain extends stub_chain_1.default {
    constructor() {
        super(...arguments);
        this._isNot = false;
        this._with = null;
        this._times = 1;
    }
    /**
     * Expect not calling.
     * @return {this} chain instance.
     */
    not() {
        this._isNot = true;
        return this;
    }
    /**
     * Expect calling times.
     * @param {Number} times Times of calling.
     * @return {this} chain instance.
     */
    times(times) {
        this._times = times;
        return this;
    }
    /**
     * Expect calling args.
     * @param {Array<Object>} args Arguments of calling.
     * @return {this} chain instance.
     */
    with(...args) {
        this._with = args;
        return this;
    }
    /**
     * Verify calling state.
     * @abstract
     * @throws on fail.
     */
    verify() {
        if (this._isNot) {
            expect(this._mock).not.toHaveBeenCalled();
            return;
        }
        expect(this._mock).toHaveBeenCalledTimes(this._times);
        if (this._with) {
            expect(this._mock).toHaveBeenCalledWith(...this._with);
        }
    }
}
exports.default = ExpectedStubChain;
//# sourceMappingURL=expected_stub_chain.js.map