"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable @typescript-eslint/explicit-module-boundary-types */
/**
 * Stub chain.
 */
class StubChain {
    /**
     * Initialize
     * @param {Object} target Stub target.
     */
    constructor(target) {
        this._target = target;
    }
    /**
     * Call original implementation.
     * @return {this} chain instance.
     */
    andCallOriginal() {
        if (this._original) {
            this._function = this._original;
        }
        else {
            throw new Error('Must call, after call #toReceive or #toReceiveMessageChain');
        }
        return this;
    }
    /**
     * Set returning value.
     * @param {Object} value returning value
     * @return {this} chain instance.
     */
    andReturn(value) {
        if (this._original) {
            this._returningValue = value;
        }
        else {
            throw new Error('Must call, after call #toReceive or #toReceiveMessageChain');
        }
        return this;
    }
    /**
     * Set execute function.
     * @param {Function} func function.
     * @return {this} chain instance.
     */
    do(func) {
        if (this._original) {
            this._function = func;
        }
        else {
            throw new Error('Must call, after call #toReceive or #toReceiveMessageChain');
        }
        return this;
    }
    /**
     * Mock method.
     * @param {string} method name of method.
     * @return {this} chain instance;
     */
    toReceive(method) {
        return this.toReceiveMessageChain(method);
    }
    /**
     * Mock method chain.
     * @param {string} methods name of method.
     * @return {this} chain instance;
     */
    toReceiveMessageChain(...methods) {
        this._original = this._target[methods[0]];
        const mocks = [];
        methods.reduce((target, method) => {
            const stub = {};
            target[method] = target[method] || jest.fn();
            if (jest.isMockFunction(target[method])) {
                target[method] = jest.fn(target[method]);
            }
            else {
                jest.spyOn(target, method);
            }
            mocks.push(target[method].mockImplementation(() => stub));
            return stub;
        }, this._target);
        this._mock = mocks[mocks.length - 1];
        // eslint-disable-next-line @typescript-eslint/no-this-alias
        const _this = this;
        this._mock.mockImplementation(function (...args) {
            if (_this._returningValue) {
                return _this._returningValue;
            }
            if (_this._function) {
                const callerContext = eval('this');
                return _this._function.bind(callerContext)(...args);
            }
        });
        return this;
    }
    /**
     * Verify calling state.
     * @abstract
     * @throws on fail.
     */
    verify() {
        // Nothing to do in StubChain.
    }
}
exports.default = StubChain;
//# sourceMappingURL=stub_chain.js.map