"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Shared Example.
 */
class SharedExample {
    constructor() {
        this._examples = {};
    }
    /**
     * Create shared example.
     * @param {string}   name     name of example.
     * @param {Function} examples example definition.
     */
    sharedExamples(name, examples) {
        this._examples[name] = examples;
    }
    /**
     * Inject shared example to current context.
     * @param {string} name name of example.
     */
    includeExamples(name, ...args) {
        const example = this._examples[name];
        if (!example) {
            throw new Error(`"${name}" is unknown shared example.`);
        }
        context(name, () => {
            example(...args);
        });
    }
}
exports.default = SharedExample;
//# sourceMappingURL=shared_example.js.map